# frozen_string_literal: true

module StackifyRubyAPM
  class Context
    # @api private
    class Response
      include NaivelyHashable

      def initialize(
        status_code,
        headers: {},
        headers_sent: true,
        finished: true
      )
        @status_code = status_code
        headers = make_xstackify_id_header(headers)
        @headers = headers
        @headers_sent = headers_sent
        @finished = finished
      end

      attr_accessor :status_code, :headers, :headers_sent, :finished

      def make_xstackify_id_header(headers)
        return unless StackifyRubyAPM.agent.current_transaction && StackifyRubyAPM.agent.current_transaction.id
        transaction_id = StackifyRubyAPM.agent.current_transaction.id
        client_id = StackifyRubyAPM.agent.config.client_id
        device_id = StackifyRubyAPM.agent.config.device_id
        headers['X-StackifyID'] = "V1|#{transaction_id}|#{client_id}|#{device_id}" if client_id && device_id
      end
    end
  end
end
