# frozen_string_literal: true

# This class builds the context of the Transaction/Span
# Context represents the HTTP request information such as headers, cookies, body.

module StackifyRubyAPM
  # @api private
  class ContextBuilder
    def initialize(_agent); end

    # Creates context
    #
    def build(rack_env)
      context = Context.new
      apply_to_request(context, rack_env)
      context
    end

    private

    # Request format and values are assigned to context
    #
    def apply_to_request(context, rack_env)
      req = rails_req?(rack_env) ? rack_env : Rack::Request.new(rack_env)
      context.request = Context::Request.new unless context.request
      request = context.request
      request.method = req.request_method
      request.url = Context::Request::Url.new(req).to_h
      request.body = get_body(req)
      request.headers = Context::Request::Headers.new(req).values

      context
    end

    def get_body(req)
      return req.POST if req.form_data?

      body = req.body.read
      req.body.rewind
      body
    end

    def rails_req?(env)
      # Not used
      # Possible use of ActiveSupport.on_load(:action_dispatch_request)
      defined?(ActionDispatch::Request) &&
        env.is_a?(ActionDispatch::Request)
    end
  end
end
