# frozen_string_literal: true

# This module build errors/exceptions and the Frames/stacktraces affected/related to the error

module StackifyRubyAPM
  # @api private
  class ErrorBuilder
    def initialize(agent)
      @agent = agent
    end

    def build_exception(exception, handled: true)
      error = Error.new
      error.exception = Error::Exception.new(exception, handled: handled)

      if exception.backtrace
        add_stacktrace error, :exception, exception.backtrace
      end

      add_transaction_id error

      if (transaction = StackifyRubyAPM.current_transaction)
        error.context = transaction.context.dup
      end

      error
    end

    def build_log(message, backtrace: nil, **attrs)
      error = Error.new
      error.log = Error::Log.new(message, **attrs)

      add_stacktrace error, :log, backtrace if backtrace
      add_transaction_id error

      error
    end

    private

    def add_stacktrace(error, kind, backtrace)
      stacktrace =
        @agent.stacktrace_builder.build(backtrace, :error)
      return unless stacktrace

      case kind
      when :exception
        error.exception.stacktrace = stacktrace
      when :log
        error.log.stacktrace = stacktrace
      end

      error.culprit = stacktrace.frames.first.function
    end

    def add_transaction_id(error)
      return unless (transaction = StackifyRubyAPM.current_transaction)
      error.transaction_id = transaction.id
    end
  end
end
