# frozen_string_literal: true

# This module gathers the built Errors and Exceptions into hash formats

module StackifyRubyAPM
  module Serializers
    # @api private
    class Errors < Serializer
      def build(error)
        if (exception = error.exception)
          current_timestamp = error.timestamp
          base = {
            CaughtBy: exception.module != '' ? exception.module : exception.type,
            Exception: exception.type,
            Message: exception.message,
            Timestamp: current_timestamp.round.to_s,
            Frames: exception.stacktrace.to_a
          }
        end

        base
      end

      def build_all(errors)
        { exceptions: Array(errors).map(&method(:build)) }
      end

      private

      def build_log(log)
        {
          message: log.message,
          level: log.level,
          logger_name: log.logger_name,
          param_message: log.param_message,
          stacktrace: log.stacktrace.to_a
        }
      end
    end
  end
end
