# frozen_string_literal: true

#
# This modules creates and initializes new Span
#

require 'securerandom'
require 'stackify_apm/span/context'
module StackifyRubyAPM
  # @api private
  class Span
    DEFAULT_TYPE = 'custom'.freeze

    # rubocop:disable Metrics/ParameterLists
    def initialize(
      transaction,
      id,
      name,
      type = nil,
      parent_id: nil,
      context: nil,
      http_status: nil
    )
      @transaction = transaction
      @id = id
      @name = name
      @type = type || DEFAULT_TYPE
      @parent_id = parent_id
      @context = context
      @http_status = http_status
      @stacktrace = nil
      @original_backtrace = nil
    end
    # rubocop:enable Metrics/ParameterLists

    attr_accessor :name, :type, :original_backtrace, :http_status
    attr_reader :id, :context, :stacktrace, :duration, :parent_id, :relative_start, :relative_end

    def start
      @relative_start = Time.now.to_f * 1000

      self
    end

    def done
      @relative_end = Time.now.to_f * 1000
      @duration = Time.now.to_f * 1000
      self.original_backtrace = nil # release it

      self
    end

    def done?
      # !!duration
      !duration.nil?
    end

    def running?
      relative_start && !done?
    end

    def inspect
      "<StackifyRubyAPM::Span id:#{id}" \
        " name:#{name.inspect}" \
        " type:#{type.inspect}" \
        '>'
    end
  end
end
