# frozen_string_literal: true

module StackifyRubyAPM
  class Span
    # @api private
    class Context
      include NaivelyHashable

      def initialize(**args)
        args.each do |key, val|
          send(:"#{key}=", val)
        end
      end

      attr_accessor :CATEGORY,
                    :SUBCATEGORY,
                    :COMPONENT_CATEGORY,
                    :COMPONENT_DETAIL,
                    :SQL,
                    :PREFIX_SQL_PARAMETERS,
                    :PREFIX_SQL_PARAMETER_COUNT,
                    :SQL_TRUNCATED,
                    :MONGODB_COLLECTION,
                    :METHOD,
                    :URL,
                    :STATUS,
                    :PROVIDER,
                    :OPERATION,
                    :CACHEKEY,
                    :CACHENAME,
                    :THREAD_ID,
                    :ID,
                    :TRACKED_FUNC,
                    :LEVEL,
                    :MESSAGE,
                    :EXCEPTION,
                    :PREFIX_REQUEST_BODY,
                    :PREFIX_REQUEST_SIZE_BYTES,
                    :PREFIX_REQUEST_HEADERS,
                    :PREFIX_RESPONSE_BODY,
                    :PREFIX_RESPONSE_SIZE_BYTES,
                    :PREFIX_RESPONSE_HEADERS,
                    :PREFIX,
                    :ACTION
      # rubocop:disable Style/VariableName
      def update_status(status)
        @STATUS = status
      end

      def update_request_body(body)
        @PREFIX_REQUEST_BODY = body.to_s
        @PREFIX_REQUEST_SIZE_BYTES = body.to_s.length.to_s
      end

      def update_request_headers(headers)
        @PREFIX_REQUEST_HEADERS = to_json_list(headers)
      end

      def update_response_body(body)
        @PREFIX_RESPONSE_BODY = body.to_s
        @PREFIX_RESPONSE_SIZE_BYTES = body.to_s.length.to_s
      end

      def update_response_headers(headers)
        @PREFIX_RESPONSE_HEADERS = to_json_list(headers)
      end

      private
      def to_json_list(header)
        list_header = []
        header.each do |key, value|
          list_header << {key => value}
        end
        list_header.to_json
      end
      # rubocop:enable Style/VariableName
    end
  end
end
