# frozen_string_literal: true

# Monkey patch for the ActionDispatch::ShowExceptions class which handles exception events.

module StackifyRubyAPM
  # @api private
  module Spies
    # @api private
    class ActionDispatchSpy
      def install
        ::ActionDispatch::ShowExceptions.class_eval do
          alias_method 'render_exception_without_apm', 'render_exception'

          def render_exception(env, exception)
            # Creates exception log report
            #
            begin
              StackifyRubyAPM.report(exception)
            rescue Exception => e
              StackifyRubyAPM.agent.error '[ActionDispatchSpy] Error: repoting exception.'
              StackifyRubyAPM.agent.error "[ActionDispatchSpy] #{e.inspect}"
            end
            render_exception_without_apm env, exception
          end
        end
      end
    end

    # Registers ActionDispatch spy, go to: /stackify_apm/spies.rb
    #
    register(
      'ActionDispatch::ShowExceptions',
      'action_dispatch/show_exception',
      ActionDispatchSpy.new
    )
  end
end
