# frozen_string_literal: true

# Monkey patch for the Log4r::Outputter class for logging log message.
#
module StackifyRubyAPM
  module Spies
    class Log4rSpy
      def install
        Log4r::Outputter.module_eval do
          alias_method 'canonical_log_without_apm', 'canonical_log'

          # Log message formatted to a logevent object
          def canonical_log(logevent)
            return canonical_log_without_apm(logevent) unless StackifyRubyAPM.current_transaction

            begin
              name = 'log4r'
              type = 'ext.log4r'
              log_message = ''
              exception = nil
              msg = logevent.data

              case msg
              when ::String
                log_message = msg
              when ::Exception
                log_message = msg.message
                exception = "(#{ msg.class })\n#{ msg.backtrace.join("\n") if msg.backtrace }"
              else
                log_message = msg.inspect
              end

              ctx = Span::Context.new(
                CATEGORY: 'Log',
                SUBCATEGORY: 'Log4r',
                LEVEL: Log4r::LNAMES[logevent.level] || 'ANY',
                MESSAGE: log_message,
                PREFIX: 'TRUE'
              )

              if exception
                ctx.EXCEPTION = exception
              end
            rescue Exception => e
              StackifyRubyAPM.agent.error "[Log4rSpy] Error: creating span context."
              StackifyRubyAPM.agent.error "[Log4rSpy] #{e.inspect}"
              return canonical_log_without_apm(logevent)
            end

            StackifyRubyAPM.span name, type, context: ctx do
              canonical_log_without_apm(logevent)
            end
          end
        end
      end
    end

    register 'Log4r::Outputter', 'log4r', Log4rSpy.new
  end
end
