# frozen_string_literal: true

# Monkey patch for the sidekiq class for running async tasks.
#

module StackifyRubyAPM
  # @api private
  module Spies
    # @api private
    class SidekiqClientSpy
      def install
        Sidekiq::Processor.class_eval do
          alias_method 'process_without_apm', 'process'

          def process(work, *args, &block)
            ret = nil
            begin
              job = nil
              if defined? work.message
                job = work.message
              else
                job = work.job
              end
              job_hash = JSON.parse job
              name = job_hash["class"]
              transaction = StackifyRubyAPM.transaction name, 'TASK'
              ret = process_without_apm(work, *args, &block)
            rescue StackifyRubyAPM::InternalError
              raise # Don't report StackifyRubyAPM errors
            rescue StandardError => e
              StackifyRubyAPM.report e
              raise e
            ensure
              transaction.submit()
            end
            ret
          end
        end
      end
    end

    register 'Sidekiq::Processor', 'sidekiq/processor', SidekiqClientSpy.new
  end
end
