# frozen_string_literal: true

# Monkey patch for the Sinatra::Base class which is responsible for handling requests.

module StackifyRubyAPM
  # @api private
  module Spies
    # @api private
    class SinatraSpy
      def install
        ::Sinatra::Base.class_eval do
          alias_method 'dispatch_without_apm!', 'dispatch!'
          alias_method 'compile_template_without_apm', 'compile_template'

          # Sets transaction name from Sinatra env's route name
          #
          def dispatch!(*args, &block)
            dispatch_without_apm!(*args, &block).tap do
              next unless (transaction = StackifyRubyAPM.current_transaction)
              next unless (route = env['sinatra.route'])

              transaction.name = route
            end
          end

          # Tilt engine template
          #
          def compile_template(engine, data, opts, *args, &block)
            opts[:__stackify_apm_template_name] =
              case data
              when Symbol then data.to_s
              else format('Inline %s', engine)
              end

            compile_template_without_apm(engine, data, opts, *args, &block)
          end
        end
      end
    end

    # Registers Sinatra spy, go to: /stackify_apm/spies.rb
    #
    register 'Sinatra::Base', 'sinatra/base', SinatraSpy.new

    # Tilt template helper & span creator
    #
    require 'stackify_apm/spies/tilt'
  end
end
