# frozen_string_literal: true

# Spies for active record when sinatra framework is used and active_record is being extended. (mysql adapter)
# Not used
# Possible use of ActiveSupport.on_load(:active_record)

require 'stackify_apm/helper/database_helper'

module StackifyRubyAPM
  # @api private
  module Spies
    # @api private
    class SqliteAdapterSpy
      TYPE = 'db.sinatra_active_record.sql'.freeze

      def install
        ActiveRecord::ConnectionAdapters::SQLite3Adapter.class_eval do
          alias_method 'exec_query_without_apm', 'exec_query'

          # rubocop:disable Lint/UnusedMethodArgument
          def exec_query(sql, name = 'SQL', binds = [], prepare: false)
            result = nil

            unless StackifyRubyAPM.current_transaction
              exec_query_without_apm(sql, name, binds)
            end

            payload = {sql: sql, binds: binds}
            statement = query_variables(payload)
            check_prepared_stmt(statement, payload)

            ctx = Span::Context.new(statement)
            result = exec_query_without_apm(sql, name, binds)

            StackifyRubyAPM.span name, TYPE, context: ctx do
              return result
            end
          end
          # rubocop:enable Lint/UnusedMethodArgument

          def query_variables(payload)
            props = get_common_db_properties
            props[:SQL] = payload[:sql]
            props
          end

          def check_prepared_stmt(statement, payload)
            if StackifyRubyAPM.agent.config.prefix_enabled
              check_prepared_stmt_by_placeholder(payload[:sql].include?('?'), statement, payload)
            end
          end
        end
      end
    end

    register 'ActiveRecord::ConnectionAdapters::SQLite3Adapter', 'active_record/connection_adapters/sqlite3_adapter', SqliteAdapterSpy.new
  end
end
