# frozen_string_literal: true

#
# Monkey patch for the Tilt::Template class on template rendering and span creation.
#

module StackifyRubyAPM
  # @api private
  module Spies
    # @api private
    class TiltSpy
      TYPE = 'template.tilt'.freeze

      def install
        ::Tilt::Template.class_eval do
          alias_method 'render_without_apm', 'render'

          def render(*args, &block)
            name = options[:__stackify_apm_template_name] || 'Unknown template'

            # Creates new span for Tilt templating
            #
            StackifyRubyAPM.span name, TYPE do
              render_without_apm(*args, &block)
            end
          end
        end
      end
    end

    # Registers Tilt spy, go to: /stackify_apm/spies.rb
    #
    register 'Tilt::Template', 'tilt/template', TiltSpy.new
  end
end
