# frozen_string_literal: true

# This class generates/appends the json log file after every Web Request

require 'stackify_apm/root_info'
require 'stackify_apm/serializers'

module StackifyRubyAPM
  # @api private
  class TraceLogger
    include Log

    def initialize(config)
      @config = config
      @trace_file_counter = 0
      begin
        @transaction_serializers = Serializers::Transactions.new(config)
        @transport = StackifyRubyAPM::TransportSelector.new(@config).transport
      rescue StandardError => exception
        debug '[TraceLogger] initialize()' if ENV['STACKIFY_TRANSPORT_LOG_LEVEL'] == '0'
        debug exception.inspect
      end
    end

    attr_accessor :trace_file_counter

    def post(transactions = [])
      @transport.post(transactions)
    end
  end
end
