require 'json'
require 'stackify_apm/root_info'
require 'stackify_apm/serializers'

module StackifyRubyAPM
  # This class will take care of building a protobuf message.
  # @api private
  class AgentBaseTransport
    include Log

    def initialize(config)
      @config = config
      @transaction_serializers = Serializers::Transactions.new(config)
    end

    def get_protobuf_message(transactions)
      protobuf_obj = build_message(transactions)
      StackifyProtoBuf::Traces.encode(protobuf_obj)
    end

    def get_json_message(transactions)
      JSON.generate(build_json_message(transactions))
    end

    # This method will build a group of Stackify::Traces from the protobuf objects.
    # It accept Array of transactions.
    def build_message(transactions = [])
      # convert transactions to protobuf
      traces = StackifyProtoBuf::Traces.new
      transactions.each do |transaction|
        # convert transaction to protobuf
        protobuf_transaction = @transaction_serializers.build_protobuf(@config, transaction)
        # add to traces array
        traces.traces.push(protobuf_transaction)
      end
      return traces
    rescue StandardError => e
      debug "[AgentBaseTransport] build_message() exception: #{e.inspect}"
    end

    def build_json_message(transactions = [])
      traces = []
      transactions.each do |transaction|
        # convert transaction to json
        json_transaction = @transaction_serializers.build_json(@config, transaction)
        # add to traces array
        traces << json_transaction
      end
      return traces
    rescue StandardError => e
      debug "[AgentBaseTransport] build_json_message() exception: #{e.inspect}"
    end

    def post(_transactions = [])
      raise NotImplementedError
    end
  end
end
