# frozen_string_literal: true

require 'stackify_apm/root_info'
require 'stackify_apm/serializers'

module StackifyRubyAPM
  # This class will handle the writing of messages through a logfile.
  # @api private
  class LogClient
    include Log

    def initialize(config)
      @config = config
      @transaction_serializers = Serializers::Transactions.new(@config)
    end

    # This method will build an Array of Transactions in a json format.
    # It will accept Array of transactions.
    def post(transactions = [])
      # convert transactions to json
      json_traces = []
      transactions.each  do |transaction|
        # convert transaction to json
        json_transaction = @transaction_serializers.build_json(@config, transaction).to_json

        # add to json traces array
        json_traces.push(json_transaction)
      end

      current_datetime = Time.now.utc.strftime('%Y-%m-%d, %H:%M:%S.%6N')

      return unless ENV['STACKIFY_RUBY_ENV'] != 'rspec'

      json_traces.each do |json_trace|
        @config.tracer_logger.<<("#{current_datetime} > #{json_trace} \n")
      end
      debug '[LogClient] post() Successfully write to logfile.' if ENV['STACKIFY_TRANSPORT_LOG_LEVEL'] == '0'
    rescue StandardError => e
      debug "[LogClient] post() exception: #{e.inspect}"
    end
  end
end
