require 'stackify_ruby_apm'

module AWS
  def self.instrument
    config = {
      transport: 'logging',
      queue: false,
    }
    StackifyRubyAPM.start config
  end

  def self.handler(event:, context:, &block)
    begin
      ctx = StackifyRubyAPM::Context.new
      ctx.add_aws_context({:arn => context.invoked_function_arn})
      transaction = StackifyRubyAPM.transaction context.function_name, 'TASK', context: ctx
      ret = block.call
    rescue StackifyRubyAPM::InternalError
      raise # Don't report StackifyRubyAPM errors
    rescue StandardError => e
      StackifyRubyAPM.report e
      raise e
    ensure
      transaction.submit()
    end
    ret
  end

  # STACKIFY LAMBDA HANDLER
  #
  # @return original function execution
  def self.stackify_handler(event:, context:)
    begin
      if !StackifyRubyAPM.running?
        config = {
          transport: 'logging',
          queue: false,
        }
        StackifyRubyAPM.start config
      end

      ctx = StackifyRubyAPM::Context.new
      ctx.add_aws_context({:arn => context.invoked_function_arn})
      transaction = StackifyRubyAPM.transaction context.function_name, 'TASK', context: ctx

      lambda_handler = StackifyRubyAPM.agent.config.lambda_handler.split('.')
      function_file = lambda_handler[0]
      function_name = lambda_handler[-1]

      require "./#{function_file}"

      send(function_name, event: event, context: context)
    rescue StackifyRubyAPM::InternalError
      raise # Don't report StackifyRubyAPM errors
    rescue StandardError => e
      StackifyRubyAPM.report e
      raise e
    ensure
      transaction.submit()
    end
  end
end
